@@ -1,67 +1,76 @@
 from typing import Any, Callable, List, Optional, Dict, Union
 
-import numpy as np
-import numpy.typing as npt
+import numpy as np  # type: ignore
 from dataclasses import dataclass, field
 
+from code_loader.contract.decoder_classes import LeapImage, LeapText, LeapNumeric, LeapGraph
 from code_loader.contract.enums import DataStateType, DatasetInputType, DatasetOutputType, DatasetMetadataType, \
     DataStateEnum
 
 
 @dataclass
-class SubsetResponse:
+class PreprocessResponse:
     length: int
     data: Any
 
 
-SectionCallableInterface = Callable[[int, SubsetResponse], np.ndarray]
-MetadataSectionCallableInterface = Union[
-    Callable[[int, SubsetResponse], int],
-    Callable[[int, SubsetResponse], str],
-    Callable[[int, SubsetResponse], bool],
-    Callable[[int, SubsetResponse], float]
+SectionCallableInterface = Callable[[int, PreprocessResponse], np.ndarray]
+
+
+@dataclass
+class PreprocessHandler:
+    function: Callable[[], List[PreprocessResponse]]
+    data_length: Dict[DataStateType, int] = field(default_factory=dict)
+
+
+DecoderCallableInterface = Union[
+    Callable[[np.array], LeapImage],
+    Callable[[np.array], LeapNumeric],
+    Callable[[np.array], LeapText],
+    Callable[[np.array], LeapGraph]
 ]
 
 
 @dataclass
-class SubsetHandler:
-    function: Callable[[], List[SubsetResponse]]
+class DecoderHandler:
     name: str
-    data_length: Dict[DataStateType, int] = field(default_factory=dict)
+    function: DecoderCallableInterface
+    heatmap_function: Optional[Callable[[np.array], np.array]] = None
 
 
 @dataclass
 class DatasetBaseHandler:
     name: str
     function: SectionCallableInterface
-    subset_name: str
 
 
 @dataclass
 class InputHandler(DatasetBaseHandler):
     type: DatasetInputType
+    decoder_name: str
     shape: Optional[List[int]] = None
 
 
+
 @dataclass
 class GroundTruthHandler(DatasetBaseHandler):
     type: DatasetOutputType
+    decoder_name: str
     labels: Optional[List[str]] = None
     masked_input: Optional[str] = None
     shape: Optional[List[int]] = None
 
 
+
 @dataclass
-class MetadataHandler:
-    name: str
-    function: MetadataSectionCallableInterface
-    subset_name: str
+class MetadataHandler(DatasetBaseHandler):
     type: DatasetMetadataType
 
 
 @dataclass
 class DatasetIntegrationSetup:
-    subsets: List[SubsetHandler] = field(default_factory=list)
+    preprocess: Optional[PreprocessHandler] = None
+    decoders: List[DecoderHandler] = field(default_factory=list)
     inputs: List[InputHandler] = field(default_factory=list)
     ground_truths: List[GroundTruthHandler] = field(default_factory=list)
     metadata: List[MetadataHandler] = field(default_factory=list)
@@ -69,8 +78,8 @@ class DatasetIntegrationSetup:
 
 @dataclass
 class DatasetSample:
-    inputs: Dict[str, npt.NDArray[np.float32]]
-    gt: Dict[str, npt.NDArray[np.float32]]
-    metadata: Dict[str, Union[str, int, bool, float]]
+    inputs: Dict[str, np.ndarray]
+    gt: Dict[str, np.ndarray]
+    metadata: Dict[str, np.ndarray]
     index: int
     state: DataStateEnum
